/*
********************************************************************************
**	Hercules I/O
********************************************************************************
**
   Copyright (C) 1995 by Jochen Hoenicke,
**			      [jochen.hoenicke@informatik.uni-oldenburg.de]
**
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
**
********************************************************************************
*/

#undef DONTSET

/*
**     GGI includes
*/

#include <kgi/kgi.h>

static inline kgiu8 inbCRT(kgiu8 reg)
	{ outb_p(reg, CRTIm); return inb_p(CRTDm); }

#ifndef DONTSET 
static inline void outbCTRL(kgiu8 value)
	{ outb_p(value, CTRLm); }

static inline void outbCONF(kgiu8 value)
	{ outb_p(value, CONFm); }

static inline void outbCRT(kgiu8 value, kgiu8 reg) 
	{ outb_p(reg, CRTIm); outb_p(value, CRTDm); }
#else
#warning DONTSET: Resister setting disabled
static inline void outbCTRL(kgiu8 value) {}
static inline void outbCONF(kgiu8 value) {}
static inline void outbCRT(kgiu8 value, kgiu8 reg) {}
#endif /* DONTSET */

/*
**	clock I/O functions
*/

void CLK_out8(struct kgi_display *dpy, kgiu8 value, kgiu8 reg)
{
	/* The HGC has only one clock */
	ASSERT(value == 0);
}

kgiu8 CLK_in8(struct kgi_display *dpy, kgiu8 reg)
{
	return 0;
}

#define	NoCRTRegs	0x10

struct statetable
{
   kgiu8 CTRL, CONF;
   kgiu8 CRT[NoCRTRegs];
};

static void restore_CRT(kgiu8 *buf)
{
	register kgiu8 i;

        for(i = 0; i < NoCRTRegs; i++) {
                outbCRT(*(buf++), i);
        }
}

static void restore_state(struct statetable *state)
{
#ifndef DONTSET
        outbCTRL(state->CTRL & ~CTRL_ENB_CRT);
        outbCONF(state->CONF);
        restore_CRT(state->CRT);
        outbCTRL(state->CTRL);
#endif
}

#if defined(DEBUG_IO)||defined(DEBUG_STATE)
static void dump_state(struct statetable *state)
{ 
        int i;
        printk("CTRL: %.2x\n",state->CTRL);
        printk("CONF: %.2x",state->CONF);
        
        for(i = 0; i < NoCRTRegs; i++) {
                if(!(i & 0xf)) printk("\nCRT:%.2x: ",i); 
                printk("%.2x, ",state->CRT[i]); }
        printk("\n");
}
#endif

